@@ -3,12 +3,14 @@ import logging
 import os
 import sys
 
-from octue.logging_handlers import get_default_handler
+from octue.logging_handlers import apply_log_handler
+from octue.resources import Child
 from octue.resources.analysis import CLASS_MAP, Analysis
 from octue.utils import gen_uuid
 from twined import Twine
 
 
+package_logger = logging.getLogger("octue")
 module_logger = logging.getLogger(__name__)
 
 
@@ -38,23 +40,35 @@ class Runner:
     """
 
     def __init__(
-        self, twine="twine.json", configuration_values=None, configuration_manifest=None, log_level=logging.INFO,
+        self,
+        twine="twine.json",
+        configuration_values=None,
+        configuration_manifest=None,
+        log_level=logging.INFO,
+        handler=None,
+        show_twined_logs=False,
     ):
         """ Constructor for the Runner class. """
+        # Store the log level (same log level used for all analyses)
+        self._log_level = log_level
+        self.handler = handler
+
+        if show_twined_logs:
+            apply_log_handler(logger=package_logger, handler=self.handler)
 
         # Ensure the twine is present and instantiate it
-        self.twine = Twine(source=twine)
+        if isinstance(twine, Twine):
+            self.twine = twine
+        else:
+            self.twine = Twine(source=twine)
 
-        if "configuration_values" not in self.twine.available_strands:
-            configuration_values = None
-
-        if "configuration_manifest" not in self.twine.available_strands:
-            configuration_manifest = None
+        package_logger.debug("Parsed twine with strands %r", self.twine.available_strands)
 
         # Validate and initialise configuration data
         self.configuration = self.twine.validate(
             configuration_values=configuration_values, configuration_manifest=configuration_manifest, cls=CLASS_MAP,
         )
+        package_logger.debug("Configuration validated.")
 
         # Set path for configuration manifest.
         # TODO this is hacky, we need to rearchitect the twined validation so we can do this kind of thing in there
@@ -64,33 +78,15 @@ class Runner:
 
         # Store the log level (same log level used for all analyses)
         self._log_level = log_level
+        self.handler = handler
 
-    def _get_analysis_logger(self, analysis_id, handler=None):
-        """ Create a logger specific to the analysis
-
-        :parameter analysis_id: The id of the analysis to get the log for. Should be unique to the analysis
-        :type analysis_id: str
-
-        :parameter handler: The handler to use. If None, default console handler will be attached.
-
-        :return: logger named in the pattern `analysis-{analysis_id}`
-        :rtype logging.Logger
-        """
-        handler = handler or get_default_handler(log_level=self._log_level)
-        analysis_logger = logging.getLogger(f"analysis-{analysis_id}")
-        analysis_logger.addHandler(handler)
-        analysis_logger.setLevel(self._log_level)
-
-        if type(analysis_logger.handlers[0]).__name__ == "SocketHandler":
-            local_logger = logging.getLogger(__name__)
-            local_logger.addHandler(get_default_handler(log_level=self._log_level))
-            local_logger.setLevel(self._log_level)
-            local_logger.info(
-                f"Logs streaming to {analysis_logger.handlers[0].host + ':' + str(analysis_logger.handlers[0].port)}"
+        if show_twined_logs:
+            apply_log_handler(logger=package_logger, handler=self.handler, log_level=self._log_level)
+            package_logger.info(
+                "Showing package logs as well as analysis logs (the package logs are recommended for software "
+                "engineers but may still be useful to app development by scientists."
             )
 
-        return analysis_logger
-
     @staticmethod
     def _update_manifest_path(manifest, pathname):
         """ A Quick hack to stitch the new Pathable functionality in the 0.1.4 release into the CLI and runner.
@@ -116,7 +112,6 @@ class Runner:
         self,
         app_src,
         analysis_id=None,
-        handler=None,
         input_values=None,
         input_manifest=None,
         credentials=None,
@@ -161,9 +156,6 @@ class Runner:
 
         :return: None
         """
-        if "input_manifest" not in self.twine.available_strands:
-            input_manifest = None
-
         inputs = self.twine.validate(
             input_values=input_values,
             input_manifest=input_manifest,
@@ -173,6 +165,13 @@ class Runner:
             allow_missing=False,
             allow_extra=False,
         )
+        package_logger.debug("Inputs validated.")
+
+        if inputs["children"] is not None:
+            inputs["children"] = {
+                child["key"]: Child(name=child["key"], id=child["id"], backend=child["backend"])
+                for child in inputs["children"]
+            }
 
         # TODO this is hacky, we need to rearchitect the twined validation so we can do this kind of thing in there
         inputs["input_manifest"] = self._update_manifest_path(inputs.get("input_manifest", None), input_manifest,)
@@ -185,7 +184,8 @@ class Runner:
         )
 
         analysis_id = str(analysis_id) if analysis_id else gen_uuid()
-        analysis_logger = self._get_analysis_logger(analysis_id, handler)
+        analysis_logger = logging.getLogger(f"analysis-{analysis_id}")
+        apply_log_handler(logger=analysis_logger, handler=self.handler, log_level=self._log_level)
 
         analysis = Analysis(
             id=analysis_id,
