@@ -1,9 +1,13 @@
+import copy
+import warnings
+from tests.base import BaseTestCase
+
 from octue import exceptions
 from octue.resources import Datafile, Dataset
-from ..base import BaseTestCase
+from octue.resources.filter_containers import FilterSet
 
 
-class DatafileTestCase(BaseTestCase):
+class DatasetTestCase(BaseTestCase):
     def test_instantiates_with_no_args(self):
         """ Ensures a Datafile instantiates using only a path and generates a uuid ID
         """
@@ -16,41 +20,63 @@ class DatafileTestCase(BaseTestCase):
         resource = Dataset(files=files, tags="one two")
         self.assertEqual(len(resource.files), 1)
 
-    def test_append_single_file_to_empty_dataset(self):
-        """ Ensures that when a dataset is empty, it can be appended to
+    def test_len(self):
+        """ Test that the length of a Dataset is the number of files it contains. """
+        dataset = self.create_valid_dataset()
+        self.assertEqual(len(dataset), len(dataset.files))
+
+    def test_iter(self):
+        """ Test that iterating over a Dataset is equivalent to iterating over its files. """
+        dataset = self.create_valid_dataset()
+        iterated_files = {file for file in dataset}
+        self.assertEqual(iterated_files, dataset.files)
+
+    def test_using_append_raises_deprecation_warning(self):
+        """ Test that Dataset.append is deprecated but gets redirected to Dataset.add. """
+        resource = Dataset()
+
+        with warnings.catch_warnings(record=True) as warning:
+            resource.append(Datafile(path="path-within-dataset/a_test_file.csv"))
+            self.assertEqual(len(warning), 1)
+            self.assertTrue(issubclass(warning[-1].category, DeprecationWarning))
+            self.assertIn("deprecated", str(warning[-1].message))
+            self.assertEqual(len(resource.files), 1)
+
+    def test_add_single_file_to_empty_dataset(self):
+        """ Ensures that when a dataset is empty, it can be added to
         """
         resource = Dataset()
-        resource.append(Datafile(path="path-within-dataset/a_test_file.csv"))
+        resource.add(Datafile(path="path-within-dataset/a_test_file.csv"))
         self.assertEqual(len(resource.files), 1)
 
-    def test_append_single_file_to_existing_dataset(self):
-        """ Ensures that when a dataset is not empty, it can be appended to
+    def test_add_single_file_to_existing_dataset(self):
+        """ Ensures that when a dataset is not empty, it can be added to
         """
         files = [Datafile(path="path-within-dataset/a_test_file.csv")]
         resource = Dataset(files=files, tags="one two")
-        resource.append(Datafile(path="path-within-dataset/a_test_file.csv"))
+        resource.add(Datafile(path="path-within-dataset/a_test_file.csv"))
         self.assertEqual(len(resource.files), 2)
 
-    def test_append_with_datafile_creation_shortcut(self):
-        """ Ensures that when a dataset is not empty, it can be appended to
+    def test_add_with_datafile_creation_shortcut(self):
+        """ Ensures that when a dataset is not empty, it can be added to
         """
         resource = Dataset()
-        resource.append(path="path-within-dataset/a_test_file.csv")
+        resource.add(path="path-within-dataset/a_test_file.csv")
         self.assertEqual(len(resource.files), 1)
 
-    def test_append_multiple_files(self):
-        """ Ensures that when a dataset is not empty, it can be appended to
+    def test_add_multiple_files(self):
+        """ Ensures that when a dataset is not empty, it can be added to
         """
         files = [
             Datafile(path="path-within-dataset/a_test_file.csv"),
             Datafile(path="path-within-dataset/a_test_file.csv"),
         ]
         resource = Dataset()
-        resource.append(*files)
+        resource.add(*files)
         self.assertEqual(len(resource.files), 2)
 
-    def test_cannot_append_non_datafiles(self):
-        """ Ensures that exception will be raised if appending a non-datafile object
+    def test_cannot_add_non_datafiles(self):
+        """ Ensures that exception will be raised if adding a non-datafile object
         """
 
         class NotADatafile:
@@ -58,11 +84,11 @@ class DatafileTestCase(BaseTestCase):
 
         resource = Dataset()
         with self.assertRaises(exceptions.InvalidInputException) as e:
-            resource.append(NotADatafile())
+            resource.add(NotADatafile())
 
-        self.assertIn("must be of class Datafile to append it to a Dataset", e.exception.args[0])
+        self.assertIn("must be of class Datafile to add it to a Dataset", e.exception.args[0])
 
-    def test_get_files_catches_single_underscore_mistake(self):
+    def test_filter_catches_single_underscore_mistake(self):
         """ Ensures that if the field name is a single underscore, that gets caught as an error
         """
         resource = Dataset(
@@ -71,13 +97,18 @@ class DatafileTestCase(BaseTestCase):
                 Datafile(path="path-within-dataset/a_test_file.txt"),
             ]
         )
+
         with self.assertRaises(exceptions.InvalidInputException) as e:
-            resource.get_files("name_icontains", filter_value="Test")
+            resource.files.filter("name_icontains", filter_value="Test")
 
-        self.assertIn("Field lookups should be in the form '<field_name>__'<filter_kind>", e.exception.args[0])
+        self.assertIn(
+            "Invalid filter name 'name_icontains'. Filter names should be in the form "
+            "'<attribute_name>__<filter_kind>'.",
+            e.exception.args[0],
+        )
 
-    def test_get_files_name_contains(self):
-        """ Ensures that get_files works with the name_contains and name_icontains lookups
+    def test_filter_name_contains(self):
+        """ Ensures that filter works with the name_contains and name_icontains lookups
         """
         resource = Dataset(
             files=[
@@ -85,19 +116,19 @@ class DatafileTestCase(BaseTestCase):
                 Datafile(path="path-within-dataset/a_test_file.txt"),
             ]
         )
-        files = resource.get_files("name__icontains", filter_value="Test")
+        files = resource.files.filter("name__icontains", filter_value="Test")
         self.assertEqual(2, len(files))
-        files = resource.get_files("name__icontains", filter_value="A")
+        files = resource.files.filter("name__icontains", filter_value="A")
         self.assertEqual(2, len(files))
-        files = resource.get_files("name__contains", filter_value="Test")
+        files = resource.files.filter("name__contains", filter_value="Test")
         self.assertEqual(1, len(files))
-        files = resource.get_files("name__icontains", filter_value="test")
+        files = resource.files.filter("name__icontains", filter_value="test")
         self.assertEqual(2, len(files))
-        files = resource.get_files("name__icontains", filter_value="file")
+        files = resource.files.filter("name__icontains", filter_value="file")
         self.assertEqual(2, len(files))
 
-    def test_get_files_name_with(self):
-        """ Ensures that get_files works with the name_endswith and name_startswith lookups
+    def test_filter_name_with(self):
+        """ Ensures that filter works with the name_endswith and name_startswith lookups
         """
         resource = Dataset(
             files=[
@@ -105,25 +136,25 @@ class DatafileTestCase(BaseTestCase):
                 Datafile(path="path-within-dataset/a_your_file.csv"),
             ]
         )
-        files = resource.get_files("name__startswith", filter_value="a_my")
+        files = resource.files.filter("name__starts_with", filter_value="a_my")
         self.assertEqual(1, len(files))
-        files = resource.get_files("name__startswith", filter_value="a_your")
+        files = resource.files.filter("name__starts_with", filter_value="a_your")
         self.assertEqual(1, len(files))
-        files = resource.get_files("name__startswith", filter_value="a_")
+        files = resource.files.filter("name__starts_with", filter_value="a_")
         self.assertEqual(2, len(files))
-        files = resource.get_files("name__startswith", filter_value="b")
+        files = resource.files.filter("name__starts_with", filter_value="b")
         self.assertEqual(0, len(files))
-        files = resource.get_files("name__endswith", filter_value="_file.csv")
+        files = resource.files.filter("name__ends_with", filter_value="_file.csv")
         self.assertEqual(2, len(files))
-        files = resource.get_files("name__endswith", filter_value="r_file.csv")
+        files = resource.files.filter("name__ends_with", filter_value="r_file.csv")
         self.assertEqual(1, len(files))
-        files = resource.get_files("name__endswith", filter_value="y_file.csv")
+        files = resource.files.filter("name__ends_with", filter_value="y_file.csv")
         self.assertEqual(1, len(files))
-        files = resource.get_files("name__endswith", filter_value="other.csv")
+        files = resource.files.filter("name__ends_with", filter_value="other.csv")
         self.assertEqual(0, len(files))
 
-    def test_get_files_by_tag(self):
-        """ Ensures that get_files works with tag lookups
+    def test_filter_by_tag(self):
+        """ Ensures that filter works with tag lookups
         """
         resource = Dataset(
             files=[
@@ -132,18 +163,19 @@ class DatafileTestCase(BaseTestCase):
                 Datafile(path="path-within-dataset/a_your_file.csv", tags="three all"),
             ]
         )
-        files = resource.get_files("tag__exact", filter_value="a")
+
+        files = resource.files.filter("tags__contains", filter_value="a")
         self.assertEqual(0, len(files))
-        files = resource.get_files("tag__exact", filter_value="one")
+        files = resource.files.filter("tags__contains", filter_value="one")
         self.assertEqual(1, len(files))
-        files = resource.get_files("tag__exact", filter_value="all")
+        files = resource.files.filter("tags__contains", filter_value="all")
         self.assertEqual(3, len(files))
-        files = resource.get_files("tag__startswith", filter_value="b")
+        files = resource.files.filter("tags__any_tag_starts_with", filter_value="b")
         self.assertEqual(2, len(files))
-        files = resource.get_files("tag__endswith", filter_value="3")
+        files = resource.files.filter("tags__any_tag_ends_with", filter_value="3")
         self.assertEqual(2, len(files))
-        files = resource.get_files("tag__contains", filter_value="hre")
-        self.assertEqual(1, len(files))
+        # files = resource.files.filter("tags__contains", filter_value="hre")
+        # self.assertEqual(1, len(files))
 
     def test_get_file_by_tag(self):
         """ Ensures that get_files works with tag lookups
@@ -171,8 +203,8 @@ class DatafileTestCase(BaseTestCase):
 
         self.assertIn("No files found with this tag", e.exception.args[0])
 
-    def test_get_files_by_sequence_notnone(self):
-        """ Ensures that get_files works with sequence lookups
+    def test_filter_by_sequence_not_none(self):
+        """ Ensures that filter works with sequence lookups
         """
         resource = Dataset(
             files=[
@@ -181,7 +213,7 @@ class DatafileTestCase(BaseTestCase):
                 Datafile(path="path-within-dataset/a_your_file.csv", sequence=None),
             ]
         )
-        files = resource.get_files("sequence__notnone")
+        files = resource.files.filter("sequence__is_not", None)
         self.assertEqual(2, len(files))
 
     def test_get_file_sequence(self):
@@ -193,7 +225,7 @@ class DatafileTestCase(BaseTestCase):
             Datafile(path="path-within-dataset/a_your_file.csv", sequence=None),
         ]
 
-        got_files = Dataset(files=files).get_file_sequence("name__endswith", filter_value=".csv", strict=True)
+        got_files = Dataset(files=files).get_file_sequence("name__ends_with", filter_value=".csv", strict=True)
         self.assertEqual(got_files, files[:2])
 
     def test_get_broken_file_sequence(self):
@@ -207,9 +239,9 @@ class DatafileTestCase(BaseTestCase):
             ]
         )
         with self.assertRaises(exceptions.BrokenSequenceException):
-            resource.get_file_sequence("name__endswith", filter_value=".csv", strict=True)
+            resource.get_file_sequence("name__ends_with", filter_value=".csv", strict=True)
 
-    def test_get_files_name_filters_include_extension(self):
+    def test_filter_name_filters_include_extension(self):
         """ Ensures that filters applied to the name will catch terms in the extension
         """
         files = [
@@ -217,9 +249,11 @@ class DatafileTestCase(BaseTestCase):
             Datafile(path="path-within-dataset/a_test_file.txt"),
         ]
 
-        self.assertEqual(Dataset(files=files).get_files("name__icontains", filter_value="txt"), [files[1]])
+        self.assertEqual(
+            Dataset(files=files).files.filter("name__icontains", filter_value="txt"), FilterSet({files[1]})
+        )
 
-    def test_get_files_name_filters_exclude_path(self):
+    def test_filter_name_filters_exclude_path(self):
         """ Ensures that filters applied to the name will not catch terms in the extension
         """
         resource = Dataset(
@@ -228,5 +262,33 @@ class DatafileTestCase(BaseTestCase):
                 Datafile(path="second-path-within-dataset/a_test_file.txt"),
             ]
         )
-        files = resource.get_files("name__icontains", filter_value="second")
+        files = resource.files.filter("name__icontains", filter_value="second")
         self.assertEqual(0, len(files))
+
+    def test_using_get_files_raises_deprecation_warning(self):
+        """ Test that Dataset.get_files is deprecated but gets redirected to Dataset.files.filter. """
+        resource = Dataset(
+            files=[
+                Datafile(path="first-path-within-dataset/a_test_file.csv"),
+                Datafile(path="second-path-within-dataset/a_test_file.txt"),
+            ]
+        )
+
+        with warnings.catch_warnings(record=True) as warning:
+            filtered_files = resource.get_files("name__icontains", filter_value="second")
+            self.assertEqual(len(warning), 1)
+            self.assertTrue(issubclass(warning[-1].category, DeprecationWarning))
+            self.assertIn("deprecated", str(warning[-1].message))
+            self.assertEqual(len(filtered_files), 0)
+
+    def test_hash_value(self):
+        """ Test hashing a dataset with multiple files gives a hash of length 128. """
+        hash_ = self.create_valid_dataset().hash_value
+        self.assertTrue(isinstance(hash_, str))
+        self.assertTrue(len(hash_) == 64)
+
+    def test_hashes_for_the_same_dataset_are_the_same(self):
+        """ Ensure the hashes for two datasets that are exactly the same are the same."""
+        first_dataset = self.create_valid_dataset()
+        second_dataset = copy.deepcopy(first_dataset)
+        self.assertEqual(first_dataset.hash_value, second_dataset.hash_value)
